/*
 * hc595.c
 *
 * Created: 21-May-18 12:06:00
 *  Author: Iuliana
 */ 

#include "hc595.h"

/***************************************
Configure Connections ***ENDS***
****************************************/

//Initialize HC595 System

void HC595Init(void)
{
   //Make the Data(DS), Shift clock (SH_CP), Store Clock (ST_CP) lines output
   HC595_DDR|=((1<<HC595_SH_CP_POS)|(1<<HC595_ST_CP_POS)|(1<<HC595_DS_POS));
   HC595_DDR|=((1<<HC595_SH_CP_POS2)|(1<<HC595_ST_CP_POS2)|(1<<HC595_DS_POS2));

   HC595Write(0b00000000, 1);
   HC595Write(0b00000000, 2);

}


//Low level macros to change data (DS)lines
#define HC595DataHigh() (HC595_PORT|=(1<<HC595_DS_POS))
#define HC595DataHigh2() (HC595_PORT|=(1<<HC595_DS_POS2))

#define HC595DataLow() (HC595_PORT&=(~(1<<HC595_DS_POS)))
#define HC595DataLow2() (HC595_PORT&=(~(1<<HC595_DS_POS2)))

//Sends a clock pulse on SH_CP line
void HC595Pulse(int device)
{
   //Pulse the Shift Clock

   if (device == 1) {
	   HC595_PORT|=(1<<HC595_SH_CP_POS);//HIGH
	   HC595_PORT&=(~(1<<HC595_SH_CP_POS));//LOW
	}
	else {
	   HC595_PORT|=(1<<HC595_SH_CP_POS2);//HIGH
	   HC595_PORT&=(~(1<<HC595_SH_CP_POS2));//LOW
	}

}

//Sends a clock pulse on ST_CP line
void HC595Latch(int device)
{
   //Pulse the Store Clock
   if (device == 1) {
	   HC595_PORT|=(1<<HC595_ST_CP_POS);//HIGH
	   _delay_loop_1(1);
	   HC595_PORT&=(~(1<<HC595_ST_CP_POS));//LOW
	   _delay_loop_1(1);
   }
   else {
	   HC595_PORT|=(1<<HC595_ST_CP_POS2);//HIGH
	   _delay_loop_1(1);
	   HC595_PORT&=(~(1<<HC595_ST_CP_POS2));//LOW
	   _delay_loop_1(1);
   }
}


/*

Main High level function to write a single byte to
Output shift register 74HC595. 

Arguments:
   single byte to write to the 74HC595 IC

Returns:
   NONE

Description:
   The byte is serially transfered to 74HC595
   and then latched. The byte is then available on
   output line Q0 to Q7 of the HC595 IC.

*/
void HC595Write(uint8_t data, int device)
{
   //Send each 8 bits serially

   //Order is MSB first
   for(uint8_t i=0;i<8;i++)
   {
      //Output the data on DS line according to the
      //Value of MSB
      if(data & 0b10000000)
      {
         //MSB is 1 so output high
		 if (device == 1)
			HC595DataHigh();
		else
			HC595DataHigh2();
      }
      else
      {
         //MSB is 0 so output high
		 if (device == 1)
			HC595DataLow();
		 else
			HC595DataLow2();
      }

      HC595Pulse(device);  //Pulse the Clock line
      data=data<<1;  //Now bring next bit at MSB position

   }

   //Now all 8 bits have been transferred to shift register
   //Move them to output latch at one
   HC595Latch(device);
}

/*

Simple Delay function approx 0.5 seconds

*/

void Wait(void)
{
/*   for(uint8_t i=0;i<30;i++)
   {
      _delay_loop_2(0);
   }*/

   _delay_ms(2000);
}