/*
 * Piano.c
 *
 * Created: 19-May-18 21:16:01
 * Author : Iuliana
 */ 

#include "hc595.h"
#include "nokia5110.h"
#include "piano.h"

#define NUM_NOTE		8

enum playMode {freePlay, learningMode, watchMode};

typedef struct {
	enum playMode md;
	char name[20];
} playModeStruct;

int nrModes = 3;

playModeStruct playModes[3];

int currentMode;

typedef struct {
	char name[40];
	int noNotes;
	char notes[70];
	char noteDuration[70];

} song;

song songs[3];
int noSongs = 3;
int requestMenu;

int frecventa_nota[NUM_NOTE] = {
	262, 294, 330, 349, 392, 440, 494, 523
};

uint8_t led_pattern[9]={
	   0b00000001,
	   0b00000010,
	   0b00000100,
	   0b00001000,
	   0b00010000,
	   0b00100000,
	   0b01000000,
	   0b10000000
   };

uint8_t turnOnAll = 0b11111111;
uint8_t turnOffAll = 0b00000000;

int currentSong;

void initPlayModes(void) {
	playModes[0].md = freePlay;
	strcpy(playModes[0].name, "Free play");
	playModes[1].md = learningMode;
	strcpy(playModes[1].name, "Practice");
	playModes[2].md = watchMode;
	strcpy(playModes[2].name, "Watch & Learn");
}




void initSongs(void) {

	strcpy(songs[0].name, "Jingle Bells");
	strcpy(songs[0].notes,        "3333333512344444333322325333333351234444433355421");
	strcpy(songs[0].noteDuration, "4424424444144444444444422442442444414444444444441");
	songs[3].noNotes = strlen(songs[3].notes);

	strcpy(songs[1].name, "Ode to Joy");
	strcpy(songs[1].notes,        "3345543211233223345543211232112231234312343212334554321123211");
	strcpy(songs[1].noteDuration, "4444444444443824444444444443824444488444884444444444444444382");
	songs[1].noNotes = strlen(songs[1].notes);

	strcpy(songs[2].name, "Twinkle Twinkle");
	strcpy(songs[2].notes,        "115566544332215544332554433211556654433221");
	strcpy(songs[2].noteDuration, "444444244444424444442444444244444424444442");
	songs[2].noNotes = strlen(songs[2].notes);

	currentSong = 2;
}

void speaker_reda(int nota) {
	int perioada_nota = 1000000l / frecventa_nota[nota];
	int i;

	/*
	 * Nu putem folosi variable ca argument pentru functia
	 * _delay_us(), asa ca folosim acest "workaround".
	 */
	PORTD |= (1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);

	PORTD &= ~(1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);
}

void freePlayMode(void)
{

	while (1) {

		while ((PINB & (1 << PB0)) == 0) {
			speaker_reda(0);
			HC595Write(led_pattern[0], 1);
		}

		while ((PINB & (1 << PB1)) == 0) {
			speaker_reda(1);
			HC595Write(led_pattern[1], 1);
		}

		while ((PINB & (1 << PB2)) == 0) {
			speaker_reda(2);
			HC595Write(led_pattern[2], 1);
		}

		while ((PINB & (1 << PB3)) == 0) {
			speaker_reda(3);
			HC595Write(led_pattern[3], 1);
		}

		while ((PINB & (1 << PB4)) == 0) {
			speaker_reda(4);
			HC595Write(led_pattern[4], 1);		
		}
		
		while ((PINB & (1 << PB5)) == 0) {
			speaker_reda(5);
			HC595Write(led_pattern[5], 1);
		}

		while ((PINB & (1 << PB6)) == 0) {
			speaker_reda(6);
			HC595Write(led_pattern[6], 1);
		}

		while ((PINB & (1 << PB7)) == 0) {
			speaker_reda(7);
			HC595Write(led_pattern[7], 1);
		}
		
	}

}

//#include <time.h>

void playSong(char *notes, char *noteDuration, int noNotes) {
	
	int i;
	for (i = 0; i < noNotes; i++) {
		int n = notes[i] - '0'; n--;
		int d = noteDuration[i] - '0';
		HC595Write(led_pattern[n], 1);
		for (int j = 0; j < 400 / d; j++)
			speaker_reda(n);
		HC595Write(turnOffAll, 1);
		_delay_ms(100);
		
	} 

}

void watchAndLearnMode(void) {

	switch (currentSong) {
		case 1:
			playSong(songs[1].notes, songs[1].noteDuration, songs[1].noNotes);
			break;
		case 2:
			playSong(songs[2].notes, songs[2].noteDuration, songs[2].noNotes);
			break;
		case 3:
			playSong(songs[3].notes, songs[3].noteDuration, songs[3].noNotes);
			break;
		default:
			break;

	}

}



void learnASong(char *notes, char *noteDuration, int noNotes) {

	int i;
	for (i = 0; i < noNotes; i++) {

		int currentNote = notes[i] - '0'; currentNote--;
		int d = noteDuration[i] - '0';

		HC595Write(led_pattern[currentNote], 1);
		if (i + 1 < noNotes) {
			int nextNote = notes[i + 1] - '0' - 1;
			HC595Write(led_pattern[nextNote], 2);
		}
		else {
			HC595Write(turnOnAll, 2);
		}
		
		while (!((PINB & (1 << currentNote)) == 0));
		
		for (int j = 0; j < 400 / d; j++)
			speaker_reda(currentNote);

		HC595Write(turnOffAll, 1);
		HC595Write(turnOffAll, 2);

		_delay_ms(100);

	}

	HC595Write(turnOnAll, 1);
	HC595Write(turnOnAll, 2);

	_delay_ms(1500);

	HC595Write(turnOffAll, 1);
	HC595Write(turnOffAll, 2);

}

void learnMode(void) {
	

	learnASong(songs[currentSong].notes, songs[currentSong].noteDuration, songs[currentSong].noNotes);


}

void changeModeMenuDisplay() {
	nokia_lcd_clear();
	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string("Current mode:", 1);
	nokia_lcd_set_cursor(0, 10);
	nokia_lcd_write_string(playModes[currentMode].name, 1);
	nokia_lcd_set_cursor(0, 20);
	nokia_lcd_write_string("Next mode: BTN1", 1);
	nokia_lcd_set_cursor(0, 30);
	nokia_lcd_write_string("Select: BTN2", 1);
	nokia_lcd_render();

}

void changeModeMenu() {
    
	changeModeMenuDisplay();
	int next = 0;

	while (next != 1) {

		if ((PIND & (1 << BTN1)) == 0) {
			currentMode = (currentMode + 1) % nrModes;
			changeModeMenuDisplay();
			while((PIND & (1 << BTN1)) == 0);
		}

		if ((PIND & (1 << BTN2)) == 0) {
			while(((PIND & (1 << BTN2)) == 0));
			next = 1;
		}


	}
}


void changeSongMenuDisplay() {
	nokia_lcd_clear();
	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string("Current song:", 1);
	nokia_lcd_set_cursor(0, 10);
	nokia_lcd_write_string(songs[currentSong].name, 1);
	nokia_lcd_set_cursor(0, 20);
	nokia_lcd_write_string("Next mode: BTN1", 1);
	nokia_lcd_set_cursor(0, 30);
	nokia_lcd_write_string("Select: BTN2", 1);
	nokia_lcd_render();

}

void changeSongMenu() {

	changeSongMenuDisplay();
	int next = 0;

	while (next != 1) {

		if ((PIND & (1 << BTN1)) == 0) {
			currentSong = (currentSong + 1) % noSongs;
			changeSongMenuDisplay();
			while((PIND & (1 << BTN1)) == 0);
		}

		if ((PIND & (1 << BTN2)) == 0) {
			while(((PIND & (1 << BTN2)) == 0));
			next = 1;
		}


	}


}

void mainMenuDisplay() {

	nokia_lcd_clear();

	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string("     Menu", 1);

	nokia_lcd_set_cursor(0, 20);
	nokia_lcd_write_string("Play: BTN1", 1);
	nokia_lcd_set_cursor(0, 30);
	nokia_lcd_write_string("Chg mode: BTN2", 1);
	nokia_lcd_set_cursor(0, 40);
	nokia_lcd_write_string("Chg song: BTN3", 1);
	
	nokia_lcd_render();

}

void mainMenu() {
	
	mainMenuDisplay();
	_delay_ms(100);

	while (1) {

			
		if ((PIND & (1 << BTN1)) == 0) {
			break;
		} 


		if ((PIND & (1 << BTN2)) == 0) {
			while(((PIND & (1 << BTN2)) == 0));
			changeModeMenu();
			mainMenuDisplay();
		}

		if ((PIND & (1 << BTN3)) == 0) {
			changeSongMenu();
			mainMenuDisplay();
		}

		_delay_ms(2);

	}
	nokia_lcd_clear();
	nokia_lcd_render();

}


int main(void)
{
	
	/* difuzor */
	DDRD |= (1 << PD4);
	PORTD &= ~(1 << PD4);

	DDRD &= ~(1 << PD0);
	DDRD &= ~(1 << PD1);
	DDRD &= ~(1 << PD5);
//	DDRD &= ~(1 << PD6);

	PORTD |= (1 << PD0);
	PORTD |= (1 << PD1);
	PORTD |= (1 << PD5);
//	PORTD |= (1 << PD6);

	 /* butoane claviatura */
	DDRB = 0x00;
	PORTB = 0xFF;

	nokia_lcd_init();
	initSongs();
	HC595Init();
	initPlayModes();

	currentMode = 1;
	requestMenu = 1;


	while (1) {




		if (requestMenu) {
			mainMenu();
			requestMenu = 0;

		}
		else {
		
			switch(playModes[currentMode].md) {
				case freePlay:
					freePlayMode();
					break;
				case watchMode:
					watchAndLearnMode();
					break;
				case learningMode:
					learnMode();
					break;
				default:
					break;


			}

		}
		
	}



}

